// Copyright 2024 Bytedance Inc. All Rights Reserved.
// Author: tianlei.richard@qq.com (tianlei.richard)

#pragma once

#include "triangle.h"
#include <opencv2/core/core.hpp>
#include <vector>

typedef struct RasterizerResult {
  double depth{-std::numeric_limits<double>::infinity()};
  int64_t triangle_index{-1};
} RasterizerResult;

class Rasterizer {
public:
  Rasterizer(const int width, const int height);

public:
  std::vector<std::vector<RasterizerResult>>
  rasterize(const std::vector<Triangle> &primitives);

  std::vector<std::vector<RasterizerResult>> get_picture() const;
  void reset();

private:
  static std::pair<bool, double> inside(const Point2d &p_screen,
                                        const Triangle &t);

private:
  int width_;
  int height_;
  std::vector<std::vector<RasterizerResult>> shading_points_;
};
